from pygame import Rect, Surface, event, mixer, MOUSEBUTTONUP
from interface import Text, Button
from data import loadimage
from constants import RESOLUTION_X, RESOLUTION_Y
from game import RunnerChar, obs_colorses, green_colors


class MenuScreen(object):

    def __init__(self, game):
        self.buf = Surface((RESOLUTION_X, RESOLUTION_Y)).convert_alpha()
        self.bg = self.buf.copy()
        self.bg.fill([0, 0, 0, 0])
        self.game = game
        self.texts = []
        self.btns = []
        self.updatables = []

    def update(self, dt):
        for t in self.texts:
            t.update(dt)
        for b in self.btns:
            b.update(dt)
        for o in self.updatables:
            o.update(dt)
        for e in event.get(MOUSEBUTTONUP):
            if e.button == 1 and not self.game.changing:
                for b in self.btns:
                    b.handleclick(e.pos)

    def draw(self, screen):
        self.buf.fill([30, 30, 30, 255])
        self.buf.blit(self.bg, (0, 0))
        for t in self.texts:
            t.draw(self.buf)
        for b in self.btns:
            b.draw(self.buf)
        for o in self.updatables:
            o.draw(self.buf)
        screen.blit(self.buf, (0, 0))

w = 264
h = 65
l = 50
bottom = 686


class MainMenu(MenuScreen):

    def __init__(self, game):
        MenuScreen.__init__(self, game)
        self.bg = loadimage("menuscreen.png").convert_alpha()
        small = 35
        self.btns = [
            Button("START", Rect(l, 190, 264, 50), game.state.select_char),
            Button("OPTIONS", Rect(l, 412, w, h - small),
                   game.state.go_options, size=16),
            Button("HOW TO PLAY", Rect(l, 462, w, h - small),
                   game.state.go_help,
                   size=16),
            Button("CREDITS", Rect(l, 512, w, h - small),
                   game.state.go_credits, size=16),
            Button("EXIT", Rect(l, 562, w, h - small), game.exit, size=16)
        ]


class Options(MenuScreen):
    def __init__(self, game):
        MenuScreen.__init__(self, game)
        self.texts = [
            Text("SOMETIMES YOU", size=30, pos=(l, 200)),
            Text("JUST DONT GET", size=30, pos=(l, 250)),
            Text("CONTROL OVER", size=30, pos=(l, 300)),
            Text("ANYTHING AT ALL", size=30, pos=(l, 350)),
            Text("BECAUSE THERE WASN'T TIME TO WRITE AN OPTIONS SCREEN",
                 pos=(l, 400))

        ]
        self.btns = [
            Button("BACK", Rect(l, bottom, w, 50), game.state.go_title)
        ]


class Credits(MenuScreen):
    def __init__(self, game):
        MenuScreen.__init__(self, game)

        self.texts = [
            Text("PYWEEK #23", size=30, pos=(l, 20)),
            Text("MS. INFORMATION", size=30, pos=(l, 50)),
            Text("PROGRAMMING, ART, MUSIC, SOUND EFFECTS",
                 size=20, pos=(l - 20, 100)),
            Text("COPYRIGHT HOPELESS OPUS CC-BY-4.0",
                 size=20, pos=(l - 20, 150)),
            Text("HOPELESS OPUS IS", size=20, pos=(l - 20, 200)),
            Text("ANDY HALSTEAD aka ASSERTIVIST", size=20, pos=(l - 20, 250)),
            Text("ANGELA PROTZMAN aka PAPERSTARSHIPS",
                 size=20, pos=(l - 20, 300)),
            Text("MIKE KRONBERGER aka MIKE KRONBERGER",
                 size=20, pos=(l - 20, 350)),
            Text("THANKS FOR", size=30, pos=(l, 400)),
            Text("PLAYING OUR GAME!!", size=30, pos=(l, 450))
        ]
        self.texts[1].hilite = True
        self.texts[-1].hilite = True
        self.texts[-2].hilite = True

        self.btns = [
            Button("BACK", Rect(l, bottom, w, h), game.state.go_title)
        ]


class Help(MenuScreen):
    def __init__(self, game):
        MenuScreen.__init__(self, game)
        self.bg = loadimage("help_bg.png")
        self.texts = [
            Text("HELP", size=30, pos=(31, 10)),
            Text("WOW, YOU'RE ON TV!!", pos=(253, 310)),
            Text("TROUBLE FLIES IN", pos=(316, 538)),
            Text("FROM THE RIGHT", pos=(316, 558)),
            Text("USE THE SPACEBAR", pos=(30, 529)),
            Text("TO JUMP. USE THE 'S'", pos=(30, 549)),
            Text("OR DOWN KEY TO DUCK.", pos=(30, 569)),
            Text("USE 'P' TO PAUSE!", pos=(160, 632))
        ]

        for t in self.texts[1:]:
            t.colors = [
                [250, 194, 255, 255],
                [237, 165, 244, 255],
                [224, 143, 231, 255]
            ]
            t.amplitude = .4
            t.hilite = True

        self.btns = [
            Button("BACK", Rect(l, bottom, w, 50), game.state.go_title)
        ]


class StageSelect(MenuScreen):
    def __init__(self, game):
        MenuScreen.__init__(self, game)

        self.texts = [
            Text("STAGE SELECT", size=30, pos=(l, 20))
        ]

        self.btns = [
            Button("STAGE 1", Rect(l, 200, w, h), self.start_game, 0),
            Button("STAGE 2", Rect(l, 280, w, h), self.start_game, 1),
            Button("STAGE 3", Rect(l, 360, w, h), self.start_game, 2),
            Button("BACK", Rect(l, bottom, w, 50), game.state.select_char)
        ]

    def start_game(self, stage):
        self.game.next_game_level = stage
        self.game.state.start_game()


class CharSelect(MenuScreen):
    def __init__(self, game):
        MenuScreen.__init__(self, game)
        self.bg = loadimage("chooseyourplayerscreen.png").convert_alpha()
        self.btns = [
            Button("MS. INFORMATION",
                   Rect(26, 276, 131, 24),
                   self.choose_char, 0,
                   17),
            Button("MR. ECTION",
                   Rect(257, 276, 131, 24),
                   self.choose_char, 1,
                   17),
            Button("BACK", Rect(l, bottom, w, 50), game.state.go_title)
        ]
        self.ms_info = RunnerChar(game, "msinformation_sprites.png")
        self.mr_ection = RunnerChar(game, "mrection_sprites.png")

        self.ms_info.char_pos = (45, 520)
        self.mr_ection.char_pos = (280, 520)

        self.updatables = [
            self.ms_info,
            self.mr_ection
        ]

    def choose_char(self, char):
        self.game.player_char = char
        self.game.state.select_stage()


class ScoreScreen(MenuScreen):
    def __init__(self, game):
        MenuScreen.__init__(self, game)
        mixer.music.stop()
        self.texts = [
            Text(
                "SCORE: {}".format(self.game.last_score),
                size=30, pos=(30, 250)),
            Text(
                "LONGEST STREAK: {}".format(self.game.last_streak),
                size=30, pos=(30, 300)),
        ]
        self.btns = [
            Button(
                "STAGE SELECTION",
                Rect(30, 600, 200, 50),
                self.game.state.select_stage,
                size=20),
            Button(
                "MAIN MENU",
                Rect(30, 670, 200, 50),
                self.game.state.go_title,
                size=20)
        ]

        if self.game.won:
            self.texts.append(Text(
                    "YOU DID IT!", size=30,
                    pos=(30, 100)))
            self.texts.append(Text(
                    "APPROVAL RATINGS ARE", size=30,
                    pos=(30, 150)))
            self.texts.append(Text(
                    "SKYHIGH!!", size=30,
                    pos=(30, 200)))
            for t in self.texts[-3:]:
                t.hilite = True
                t.colors = green_colors
        else:
            self.texts.append(Text(
                    "YOU CAN DO THIS!", size=30,
                    pos=(30, 100))),
            self.texts.append(Text(
                    "YOU'RE THE SPIN MASTER!", size=30,
                    pos=(30, 150)))
            for t in self.texts[-2:]:
                t.hilite = True
                t.colors = obs_colorses[1]

            self.btns.append(
                Button(
                    "RETRY",
                    Rect(30, 200, 200, 50),
                    self.game.state.start_game,
                    size=20))


class Pause(MenuScreen):
    def __init__(self, game):
        MenuScreen.__init__(self, game)
        self.texts = [
            Text("GAME PAUSED", size=30, pos=(30, 100))
        ]
        self.btns = [
            Button(
                "RESUME",
                Rect(30, 200, 200, 50),
                self.game.state.unpause,
                size=17
            ),
            Button(
                "STAGE SELECTION",
                Rect(30, 280, 200, 50),
                self.game.state.select_stage,
                size=17
            ),
            Button(
                "MAIN MENU",
                Rect(30, 360, 200, 50),
                self.game.state.go_title,
                size=17
            )
        ]
